//+------------------------------------------------------------------+
//|                                            Twiggs Money Flow.mq4 |
//|                                                                  |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright " Mr. Colin Twiggs"
#property link      "http://www.incrediblecharts.com/technical/twiggs_money_flow.php"

#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1  Lime

//
//
//
//
//

extern int Prd = 21;
extern int SmoothingPeriod = 5;

//
//
//
//
//

double ExtMapBuffer1[];
double WildersBuffer[];
double WVBuffer[];
double ADVBuffer[];
double SmV[];
double SmA[];


//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
//
//

int init()
{

   IndicatorBuffers(6);
   SetIndexBuffer(0,ExtMapBuffer1);
   SetIndexBuffer(1,WildersBuffer);
   SetIndexBuffer(2,WVBuffer);
   SetIndexBuffer(3,ADVBuffer);
   SetIndexBuffer(4,SmV);
   SetIndexBuffer(5,SmA);
   SetLevelValue(0,0.0);
   SetLevelValue(1,0.1);
   SetLevelValue(2,0.25);
   SetLevelValue(3,-0.1);
   SetLevelValue(4,-0.25);
   
return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int deinit()
{

   
return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars < 0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);

     //
     //
     //
     //
     //

     for (int i = limit; i >= 0; i--)
     {

       double TRH = iHigh(NULL, 0, iHighest(NULL, 0, MODE_HIGH, 1, i)) + 1*Point;
       double TRL = iLow(NULL, 0, iLowest(NULL, 0, MODE_LOW, 1, i)) - 1*Point;
       double TR  =  TRH-TRL;
       if (TR==0) { ADVBuffer[i] = ((Close[i]-TRL)-(TRH-Close[i]))/ (999999)*Volume[i];  }
       else       { ADVBuffer[i] = ((Close[i]-TRL)-(TRH-Close[i]))/ (TR)*Volume[i];      }
      
        WVBuffer[i]     = Volume[i] + (iVolume(NULL,0,i)*0);  
        WildersBuffer[i]= iMA(NULL,0,(Prd*2)-1,0,MODE_EMA,PRICE_CLOSE,i);
   
      
    }

                  
      for (i = limit; i >= 0; i--)
      {
        SmV[i] = iMAOnArray(WVBuffer,0,(Prd*2)-1,0,MODE_EMA,i);
        SmA[i]= iMAOnArray(ADVBuffer,0,(Prd*2)-1,0,MODE_EMA,i);
      
       if (TRH != 0)
       {
         ExtMapBuffer1[i] = iSsm(SmA[i]/SmV[i],SmoothingPeriod,i);
       }
   }
return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

#define Pi 3.14159265358979323846264338327950288
double workSsm[][2];
#define _tprice 0
#define _ssm    1

double workSsmCoeffs[][4];
#define _period 0
#define _c1     1
#define _c2     2
#define _c3     3

//
//
//
//
//

double iSsm(double price, double period, int i, int instanceNo=0)
{
   if (period<=1) return(price); i = Bars-i-1;
   if (ArrayRange(workSsm,0) !=Bars)                 ArrayResize(workSsm,Bars);
   if (ArrayRange(workSsmCoeffs,0) < (instanceNo+1)) ArrayResize(workSsmCoeffs,instanceNo+1);
   if (workSsmCoeffs[instanceNo][_period] != period)
   {
      workSsmCoeffs[instanceNo][_period] = period;
      double a1 = MathExp(-1.414*Pi/period);
      double b1 = 2.0*a1*MathCos(1.414*Pi/period);
         workSsmCoeffs[instanceNo][_c2] = b1;
         workSsmCoeffs[instanceNo][_c3] = -a1*a1;
         workSsmCoeffs[instanceNo][_c1] = 1.0 - workSsmCoeffs[instanceNo][_c2] - workSsmCoeffs[instanceNo][_c3];
   }

   //
   //
   //
   //
   //

      int s = instanceNo*2;   
          workSsm[i][s+_tprice] = price;
          workSsm[i][s+_ssm]    = workSsmCoeffs[instanceNo][_c1]*(workSsm[i][s+_tprice]+workSsm[i-1][s+_tprice])/2.0 + 
                                  workSsmCoeffs[instanceNo][_c2]*workSsm[i-1][s+_ssm]                                + 
                                  workSsmCoeffs[instanceNo][_c3]*workSsm[i-2][s+_ssm]; 
   return(workSsm[i][s+_ssm]);
}